# Reference
<!-- DO NOT EDIT: This document was generated by Puppet Strings -->

## Table of Contents

**Classes**

* [`system_users::delete_nis`](#system_usersdelete_nis): Delete _legacy_ NIS users from this system
* [`system_users::delete_root_alias`](#system_usersdelete_root_alias): Remove all aliases of the root account
* [`system_users::disable_system_account`](#system_usersdisable_system_account): Lock the password and shell of system/low uid users
* [`system_users::homedir`](#system_usershomedir): Ensure user homedirs set to correct `mode`
* [`system_users::lock_empty_password`](#system_userslock_empty_password): Lock all user accounts with empty passwords
* [`system_users::lock_inactive`](#system_userslock_inactive): Lock users who have been inactive for a certain period
* [`system_users::root`](#system_usersroot): Ensure root user and its homedir are setup correctly
* [`system_users::root_group`](#system_usersroot_group): Set the group for the `root` user to be GID 0

**Resource types**

* [`system_users_disable_overrides`](#system_users_disable_overrides): Disable the files .netrc, .forward, .rhosts and .shosts by changing the owner to root and the permissions to 000

## Classes

### system_users::delete_nis

Delete _legacy_ NIS users from this system (lines starting `+` in `/etc/group`, `/etc/gshadow`, `/etc/passwd` and
`/etc/shadow`).

* **See also**
https://forge.puppet.com/geoffwilliams/filemagic

#### Examples

##### Remove all `+` entries

```puppet
include system_users::delete_nis
```

### system_users::delete_root_alias

Remove all aliases of the root account (UID == `0` and username != `root`). These are
identified by the `user_audit` fact

### system_users::disable_system_account

Lock the password and shell of:

*System users*
* `system_uids` - uid < 1000

*Low uids*
* `low_uids` - uid < 500

For system accounts present on this node excluding:
  * root
  * sync (password lock only)
  * shutdown (password lock only)
  * halt (password lock only)

You can exclude particular usernames from the above requirement with the `exempt_users`
parameter. This should be used if you want to leave particular users alone. The action
of locking the password and shell is guarded to try to avoid clashes with other catalog
resources. For this to work, you need to evaluate this class _after_ any other user
resources that should take precidence.

The data on users in the UID range is sourced from the `user_audit` fact which ships inside this module. The fact
itself excludes the root user.

#### Parameters

The following parameters are available in the `system_users::disable_system_account` class.

##### `uid_range`

Data type: `Enum['low_uids', 'system_uids']`

Range of UIDs to lockdown (see above)

Default value: 'system_uids'

##### `exempt_users`

Data type: `Array[String]`

Additional usernames to exempt

Default value: []

### system_users::homedir

The `user_audit` fact contains a list of all homedirs for users local to this system. We
use this information to enforce the desired mode on these directories, excluding the `root`
user and other system home directories (see code for details).

* **Note** The `mode` parameter must be set for any changes to happen.

#### Parameters

The following parameters are available in the `system_users::homedir` class.

##### `mode`

Data type: `Optional[String]`

Mode to set home directories to, eg `0700`

Default value: `undef`

### system_users::lock_empty_password

The `user_audit` fact contains a list of users with empty passwords, we use this to
determine if any accounts need locking

#### Parameters

The following parameters are available in the `system_users::lock_empty_password` class.

##### `lock_method`

Data type: `Any`

string to insert into password field to lock the account

Default value: '*'

### system_users::lock_inactive

Lock users who have been inactive for a certain period:

*Solaris*
  1. set `definact` in `/usr/sadm/defadduser`
  2. `useradd -D`

*RHEL*
  * set `INACTIVE=` in `/etc/default/useradd`

#### Parameters

The following parameters are available in the `system_users::lock_inactive` class.

##### `period`

Data type: `Integer`

How long to wait before locking a user (days)

Default value: 30

### system_users::root

Features:
  * Manage the `root` user and its homedir
  * Manage permissions on `/root` and ensure it exists

### system_users::root_group

Manage the `gid` of the `root` user and nothing else

* **Note** incompatible with `system_users::root`

## Resource types

### system_users_disable_overrides

Disable the files .netrc, .forward, .rhosts and .shosts by changing the owner to root and the permissions to 000

#### Properties

The following properties are available in the `system_users_disable_overrides` type.

##### `ensure`

Valid values: overrides_disabled

not used

#### Parameters

The following parameters are available in the `system_users_disable_overrides` type.

##### `ensure`

namevar

arbitrary sting

